#include <Stepper.h>

/*----- Constants -----*/
#define STEPS_PER_REV 32       // Steps per revolution of the internal shaft
#define FULL_ROTATION 2048     // Steps for one full revolution
#define STEP_SIZE 0.04883         // Microns per step (adjust based on your setup)
#define BAUD_RATE 9600         // Communication baud rate

/*----- Stepper Motor Setup -----*/
Stepper small_stepper(STEPS_PER_REV, 8, 10, 9, 11); // Motor driver pins
float z = 0.0;  // Current Z position in microns

/*----- Initialization -----*/
void setup() {
  Serial.begin(BAUD_RATE);         // Initialize serial communication
  small_stepper.setSpeed(300);     // Set stepper motor speed (RPM)
  Serial.println("Vers:LS");       // Initial handshake with Micro-Manager
}

void loop() {
  if (Serial.available()) {
    String cmd = Serial.readStringUntil('\r'); // Read command until carriage return
    processCommand(cmd);
  }
}

/*----- Command Processing -----*/
void processCommand(String s) {
  if (s.startsWith("?ver")) {
    reply("Vers:LS");
  } else if (s.startsWith("!autostatus 0")) {
    delay(5);
  } else if (s.startsWith("?det")) {
    reply("60");
  } else if (s.startsWith("?pitch z")) {
    reply("50.0"); // Step size in microns
  } else if (s.startsWith("?vel z")) {
    reply("100.0"); // Maximum velocity
  } else if (s.startsWith("?accel z")) {
    reply("1.0"); // Acceleration (dummy value)
  } else if (s.startsWith("!dim z 1") || s.startsWith("!dim z 2")) {
    delay(5);
  } else if (s.startsWith("?statusaxis")) {
    reply("@");
  } else if (s.startsWith("?pos z")) {
    reply(String(z, 2)); // Report current Z position
  } else if (s.startsWith("?lim z")) {
    reply("0.0 100.0"); // Limits in microns
  } else if (s.startsWith("!mor z")) {
    // Move relative: "!mor z [delta]"
    String deltaStr = s.substring(s.indexOf("z") + 1);
    float delta = deltaStr.toFloat();
    moveRelative(delta);
  } else if (s.startsWith("!moa z")) {
    // Move absolute: "!moa z [target]"
    String targetStr = s.substring(s.indexOf("z") + 1);
    float target = targetStr.toFloat();
    moveAbsolute(target);
  } else if (s.startsWith("?status")) {
    reply("OK...");
  } else if (s.startsWith("?err")) {
    reply("0");
  } else {
    reply("Unknown Command");
  }
}

/*----- Movement Functions -----*/

// Move to an absolute position
void moveAbsolute(float targetPos) {
  if (targetPos < 0.0 || targetPos > 100.0) { // Check bounds
    reply("ERROR: Out of Bounds");
    return;
  }
  float delta = targetPos - z; // Calculate the relative move
  moveRelative(delta);
}

// Move by a relative amount
void moveRelative(float delta) {
  int steps = delta / STEP_SIZE; // Convert microns to steps
  small_stepper.step(steps);
  z += delta; // Update current Z position
  reply("OK");
}

/*----- Utility Functions -----*/

// Send a reply to Micro-Manager
void reply(String s) {
  Serial.print(s);
  Serial.print("\r"); // End response with a carriage return
}